const std = @import("std");

const build_options = @import("build_options");

const Self = @This();

pub const Dimension = i32;
pub const Coordinate = i32;

/// Maximum length of a material name
pub const material_max_len: usize = build_options.material_max_len;

/// Maximum CSV payload size for a block update
pub const csv_max_len: usize = blk: {
    const str = std.fmt.comptimePrint(
        "-{d},-{d},-{d},-{d},",
        .{
            std.math.maxInt(Dimension),
            std.math.maxInt(Coordinate),
            std.math.maxInt(Coordinate),
            std.math.maxInt(Coordinate),
        },
    );
    break :blk str.len + material_max_len;
};

dimension: Dimension,
x: Coordinate,
y: Coordinate,
z: Coordinate,

/// Material name buffer
material: [material_max_len]u8 = undefined,

/// Length of material name buffer `material`
material_len: usize = 0,

/// Converts block update message to CSV payload
/// Format: *`<dimension>,<x>,<y>,<z>,<material>`* (`<int32>,<int32>,<int32>,<int32>,<str>`)
pub fn toCsv(self: *const Self, buf: *[csv_max_len]u8) ![]u8 {
    var material_upper_buf: [material_max_len]u8 = undefined;
    const material_upper = std.ascii.upperString(&material_upper_buf, self.material[0..self.material_len]);

    return try std.fmt.bufPrint(buf, "{d},{d},{d},{d},{s}", .{
        self.dimension,
        self.x,
        self.y,
        self.z,
        material_upper,
    });
}
